import es
import playerlib
from AchievementIO import *

achievementIO = None

# == ACHIEVEMENT CLASS ==================================
class Achievement:
    """
    Class to maintain properties of a particular achievement
    """
    #"Private" Member variables
    __parentID = None
    __id = None
    __name = None
    __description = None
    __totalProgress = None
    __playerProgress = {}
    
    def __init__(self, newParentID, newID, newName, newDescription, newTotalProgress = 1, imgPath = None):
        self.__parentID = newParentID
        self.__id = newID
        self.__name = newName
        self.__description = newDescription
        self.__totalProgress = int(newTotalProgress)
        if not achievementIO is None:
            achievementIO.setAchievementProperties(newParentID, newID, newName, newDescription, newTotalProgress, imgPath)

    def __getattr__(self, name):
        #A way of ensuring member variables are "private", but can be accessed like public attributes using public names
        if name == "parentid":
            return self.__parentID
        elif name == "id":
            return self.__id
        elif name == "name":
            return self.__name
        elif name == "description":
            return self.__description
        elif name == "totalprogress":
            return int(self.__totalProgress)
        elif name == "playerprogress":
            return self.__playerProgress
        elif name in self.__dict__.keys():
            return self.__dict__[name]
        else:
            raise AttributeError, name
    
    def __setattr__(self, name, value):
        #A way of ensuring member variables are "private", but can be accessed like public attributes using public names
        if name == "id":
            self.__id = value
        elif name == "name":
            self.__name = value
        elif name == "description":
            self.__description = value
        elif name == "totalprogress":
            self.__totalProgress = int(value)
        else:
            self.__dict__[name] = value

    def setProgressForPlayer(self, playerID, progress, cacheOnly = False):
        """
        Sets the progress of this achievement for a particular player
        """
        steamID = getUniqueSteam(playerID)
        if steamID and not steamID == "None":
            self.__databaseSetProgress(steamID, progress, cacheOnly)

    def stepProgressForPlayer(self, playerID, stepAmount = 1):
        """
        Steps the progress of this achievement for a particular player by a designated step amount
        """
        backwardStep = False
        if playerID and not playerID == "None":
            steamID = getUniqueSteam(playerID)
            progress = self.__databaseGetProgress(steamID)
                
            if not progress == None:
                if progress == self.__totalProgress:
                    if stepAmount >= 0: 
                        return None
                    else:
                        backwardStep = True
                progress = progress + stepAmount
            else:
                progress = stepAmount
        
            if progress < 0:
                progress = 0
            if progress >= self.__totalProgress:
                progress = self.__totalProgress
                self.__doAchievementPlayer(playerID)
            elif backwardStep:
                self.__undoAchievementPlayer(playerID)
                
            self.__databaseSetProgress(steamID, progress)
    
    def stepProgressForList(self, listFilter, stepAmount = 1):
        """
        Steps the progress of this achievement for a particular list of players by a designated step amount
        """
        for ply in playerlib.getUseridList(listFilter):
            self.stepProgressForPlayer(ply, stepAmount)
    
    def stepProgressForAllActive(self, stepAmount = 1):
        """
        Steps the progress of this achievement for all players by a designated step amount
        """
        for ply in es.getUseridList():
            self.stepProgressForPlayer(ply, stepAmount)
            
    def completeProgressForPlayer(self, playerID):
        """
        Updates the progress of this achievement for a particular player such that the achievement becomes complete
        """
        steamID = getUniqueSteam(playerID)
        
        if steamID and not steamID == "None":
            progress = self.__databaseGetProgress(steamID)
    
            if (not progress) or progress < self.__totalProgress:
                self.__databaseSetProgress(steamID, self.__totalProgress)
                self.__doAchievementPlayer(playerID)
    
    def completeProgressForList(self, listFilter):
        """
        Updates the progress of this achievement for a particular list of players such that the achievement becomes complete
        """
        for ply in playerlib.getUseridList(listFilter):
            self.completeProgressForPlayer(ply)
                
    def completeProgressForAllActive(self):
        """
        Updates the progress of this achievement for all players such that the achievement becomes complete
        """
        for ply in es.getUseridList():
            self.completeProgressForPlayer(ply)
                
    def resetProgressForPlayer(self, playerID):
        """
        Resets the progress of this achievement for a particular player
        """
        steamID = getUniqueSteam(playerID)

        if steamID and not steamID == "None":
            progress = self.__databaseGetProgress(steamID)                
            self.__databaseSetProgress(steamID, 0)
            if progress and progress == self.__totalProgress:
                self.__resetAchievementPlayer(playerID)
       
    def resetProgressForList(self, listFilter):
        """
        Resets the progress of this achievement for a particular list of players
        """
        for ply in playerlib.getUseridList(listFilter):
            self.resetProgressForPlayer(ply, stepAmount)
                
    def resetProgressForAllActive(self):
        """
        Resets the progress of this achievement for all players
        """
        for ply in es.getUseridList():
            self.resetProgressForPlayer(ply, stepAmount)
                                          
    def getPlayerProgress(self, playerID):
        """
        Returns the current progress of this achievement for a particular player
        """
        steamID = getUniqueSteam(playerID)
        
        if steamID and not steamID == "None":
            if "STEAM" in playerID:
                steamID = playerID
            
            return self.__databaseGetProgress(steamID)
        return 0
    
    def isPlayerCompleted(self, playerID):
        """
        Returns true if the current progress of this achievement for the particular player is deemed "complete"
        """
        return self.getPlayerProgress(playerID) == self.__totalProgress

    def __doAchievementPlayer(self, playerID):
        """
        "Private" method that triggers a source event when a player achieves an achievement
        """       
        self.__triggerEvent("achievement_achieved", playerID)     
    
    def __undoAchievementPlayer(self, playerID):
        """
        "Private" method that triggers a source event when a player un-achieves an achievement
        """       
        self.__triggerEvent("achievement_unachieved", playerID)

    def __resetAchievementPlayer(self, playerID):
        """
        "Private" method that triggers a source event when a player's achievement is reset
        """       
        self.__triggerEvent("achievement_reset", playerID)
    
    def __triggerEvent(self, eventName, playerID):
        """
        "Private" method that fires a source event according to the eventName passed
        """
        es.event("initialize", eventName)
        es.event("setstring", eventName, "achievementgroupid", self.__parentID)
        es.event("setstring", eventName, "achievementid", self.__id)
        es.event("setstring", eventName, "playerid", playerID)
        es.event("fire", eventName)
    
    def __databaseGetProgress(self, steamID):
        """
        "Private" method that gets the progress of a particular player from an AchievementIO instance
        """
        if not achievementIO is None:
            return achievementIO.getPlayerProgress(self.__parentID, self.__id, steamID)
        return 0
            
    def __databaseSetProgress(self, steamID, progress, cacheOnly = False):
        """
        "Private" method that sets the progress of a particular player to an AchievementIO instance
        """
        if not achievementIO is None:
            achievementIO.setPlayerProgress(self.__parentID, self.__id, steamID, progress, cacheOnly)
# =======================================================

# == HELPER METHODS =====================================
uniqueSteams = {}

def getUniqueSteam(playerID):
    global uniqueSteams
    playerID = str(playerID)
    """
    A helper method that returns [as far as possible] a 'unique' STEAM ID.
    """
    if playerID in uniqueSteams.keys():
        return uniqueSteams[playerID]
    
    if playerID == "0" or playerID == None or playerID == "None":
        return None
    
    if "STEAM" in playerID or "BOT" in playerID:
        return playerID
        
    playerSteam = playerlib.getPlayer(playerID).attributes["steamid"]
    if "LAN" in playerSteam or "BOT" in playerSteam:
        playerSteam = playerSteam + playerlib.getPlayer(playerID).attributes["name"]

    if not achievementIO is None:
        achievementIO.setPlayerName(playerSteam, playerlib.getPlayer(playerID).attributes["name"])

    uniqueSteams[playerID] = playerSteam
    return playerSteam

def getPlayerName(playerID):
    """
    A helper method that returns the stored player name for the player of given playerID
    """
    steamID = getUniqueSteam(playerID)
    if not achievementIO is None:
        return achievementIO.getPlayerName(steamID)
    return ""

def sortDictByValue(d):
    """ Returns the keys of dictionary d sorted by their values """
    items=d.items()
    backitems=[ [v[1],v[0]] for v in items]
    backitems.sort(reverse=True)
    return [ backitems[i][1] for i in range(0,len(backitems))]      
# =======================================================